@@ -1,14 +1,14 @@
+# SPDX-License-Identifier: MIT
+# Copyright (c) 2016, Elaborated Networks GmbH
+# Copyright (c) 2019, Mathias Laurin
+
 """Three-key triple DES cipher (also known as DES3, 3DES,
 Triple DES, or DES-EDE3)."""
 
 
-__author__ = "Mathias Laurin"
-__copyright__ = "Copyright 2016, Elaborated Networks GmbH"
-__license__ = "MIT License"
-
+from mbedtls.exceptions import TLSError
 
 from . import _cipher
-from mbedtls.exceptions import *
 
 __all__ = ["block_size", "key_size", "new"]
 
