@@ -1,3 +1,6 @@
+# SPDX-License-Identifier: MIT
+# Copyright (c) 2019, Mathias Laurin
+
 """Generate secure random numbers.
 
 This is an implementation of the PEP 506 API based on a
@@ -6,16 +9,12 @@ CSPRNG (Cryptographically Strong Pseudo Random Number Generator).
 This module is compatibale with the standard `secrets` (PEP 506) module.
 
 """
-__author__ = "Mathias Laurin"
-__copyright__ = "Copyright 2019, Mathias Laurin"
-__license__ = "MIT License"
 
 import base64 as _base64
 import binascii as _binascii
 
 import mbedtls._random as _rnd
 
-
 __all__ = [
     "randbits",
     "choice",
@@ -34,13 +33,7 @@ __rng = _rnd.default_rng()
 
 randbits = __rng.getrandbits
 choice = __rng.choice
-
-
-def randbelow(upper_bound):
-    """Return a random int in the range [0, n)."""
-    if upper_bound <= 0:
-        raise ValueError("Upper bound must be positive.")
-    return __rng._randbelow(upper_bound)
+randbelow = __rng.randbelow
 
 
 def token_bytes(nbytes=None):
@@ -51,7 +44,7 @@ def token_bytes(nbytes=None):
     """
     if nbytes is None:
         nbytes = DEFAULT_ENTROPY
-    return __rng._urandom(nbytes)
+    return __rng.urandom(nbytes)
 
 
 def token_hex(nbytes=None):
