@@ -4,10 +4,6 @@
 import enum
 import socket as _socket
 import struct
-import sys
-from contextlib import suppress
-
-import mbedtls._ringbuf as _rb
 
 from ._tls import (
     DTLSConfiguration,
@@ -300,7 +296,7 @@ class TLSWrappedSocket:
         self._socket.sendall(encrypted)
 
     def sendto(self, message, *args):
-        if not 2 <= len(args) <= 3:
+        if not 1 <= len(args) <= 2:
             raise TypeError(
                 "sendto() takes 2 or 3 arguments (%i given)" % (1 + len(args))
             )
