@@ -1,6 +1,7 @@
 """Unit tests for mbedtls.pk."""
 
 import numbers
+import pickle
 
 import pytest
 
@@ -44,18 +45,22 @@ class TestECPoint:
     def point(self, xyz):
         return ECPoint(*xyz)
 
+    @pytest.mark.parametrize("repr_", (repr, str), ids=lambda f: f.__name__)
+    def test_repr(self, repr_, point):
+        assert isinstance(repr_(point), str)
+
+    def test_pickle(self, point):
+        assert point == pickle.loads(pickle.dumps(point))
+
+    def test_hash(self, point):
+        assert isinstance(hash(point), int)
+
     def test_accessors(self, point, xyz):
         x, y, z = xyz
         assert point.x == x
         assert point.y == y
         assert point.z == z
 
-    def test_str(self, point):
-        assert str(point) == "ECPoint(1, 2, 3)"
-
-    def test_repr(self, point):
-        assert repr(point) == "ECPoint(MPI(1), MPI(2), MPI(3))"
-
     def test_eq_point(self, point, xyz):
         assert (point == ECPoint(*xyz)) is True
         assert (point == ECPoint(0, 0, 0)) is False
@@ -66,12 +71,6 @@ class TestECPoint:
         assert (zero == 0) is True
         assert (zero == ECPoint(0, 0, 0)) is True
 
-    def test_hash(self, point):
-        zero = ECPoint(0, 0, 0)
-        assert hash(zero) == hash(zero)
-        assert hash(point) == hash(point)
-        assert hash(zero) != hash(point)
-
     def test_bool(self, point):
         assert bool(point) is True
         assert bool(ECPoint(0, 0, 0)) is False
@@ -90,6 +89,12 @@ class _TestCipherBase:
     def pub(self, cipher, key):
         return type(cipher).from_buffer(cipher.export_public_key())
 
+    def test_pickle(self, cipher):
+        assert cipher == pickle.loads(pickle.dumps(cipher))
+
+    def test_hash(self, cipher):
+        assert isinstance(hash(cipher), int)
+
     @pytest.mark.usefixtures("key")
     def test_cmp_eq(self, cipher):
         assert cipher == cipher
@@ -316,6 +321,12 @@ class _TestDHBase:
     def dhentity(self, modulus, generator):
         raise NotImplementedError
 
+    def test_pickle(self, dhentity):
+        with pytest.raises(TypeError) as excinfo:
+            pickle.dumps(dhentity)
+
+        assert str(excinfo.value).startswith("cannot pickle")
+
     def test_modulus(self, dhentity, modulus):
         assert dhentity.modulus == modulus
 
@@ -384,6 +395,18 @@ class TestECDH:
     def cli(self, curve):
         return ECDHClient(curve)
 
+    def test_srv_pickle(self, srv):
+        with pytest.raises(TypeError) as excinfo:
+            pickle.dumps(srv)
+
+        assert str(excinfo.value).startswith("cannot pickle")
+
+    def test_cli_pickle(self, cli):
+        with pytest.raises(TypeError) as excinfo:
+            pickle.dumps(cli)
+
+        assert str(excinfo.value).startswith("cannot pickle")
+
     def test_srv_key_accessors_without_key(self, srv):
         assert not srv._has_private()
         assert not srv._has_public()
