@@ -1,5 +1,6 @@
 """Unit tests for mbedtls.random."""
 
+import pickle
 import random
 from collections import defaultdict
 
@@ -19,6 +20,12 @@ class TestEntropy:
     def entropy(self):
         return _drbg.Random()._entropy
 
+    def test_pickle(self, entropy):
+        with pytest.raises(TypeError) as excinfo:
+            pickle.dumps(entropy)
+
+        assert str(excinfo.value).startswith("cannot pickle")
+
     def test_gather(self, entropy):
         # Only test that this does not raise.
         entropy.gather()
@@ -54,6 +61,12 @@ class TestRandom:
     def random(self):
         return _drbg.Random()
 
+    def test_pickle(self, random):
+        with pytest.raises(TypeError) as excinfo:
+            pickle.dumps(random)
+
+        assert str(excinfo.value).startswith("cannot pickle")
+
     def test_reseed(self, random):
         random._reseed()
 
